@@ -23,14 +23,16 @@
 
 #include <config.h>
 
-#include "gal-define-views-dialog.h"
-
 #include <gtk/gtk.h>
+
+#include "gal/e-table/e-table-scrolled.h"
+#include "gal/util/e-i18n.h"
+#include "gal/util/e-util.h"
+#include "gal/util/e-util-private.h"
+
+#include "gal-define-views-dialog.h"
 #include "gal-define-views-model.h"
 #include "gal-view-new-dialog.h"
-#include <gal/e-table/e-table-scrolled.h>
-#include <gal/util/e-i18n.h>
-#include <gal/util/e-util.h>
 
 static void gal_define_views_dialog_init	 (GalDefineViewsDialog		 *card);
 static void gal_define_views_dialog_class_init	 (GalDefineViewsDialogClass	 *klass);
@@ -154,7 +156,6 @@ gdvd_button_modify_callback(GtkWidget *widget, GalDefineViewsDialog *dialog)
 						       row);
 		gal_view_edit(view, GTK_WINDOW (dialog));
 	}
-
 }
 
 static void
@@ -206,16 +207,51 @@ gdvd_connect_signal(GalDefineViewsDialog *dialog, char *widget_name, char *signa
 		g_signal_connect(widget, signal, handler, dialog);
 }
 
+static void
+etable_selection_change_forall_cb (int row, GalDefineViewsDialog *dialog)
+{
+	if (row != -1) {
+		GalViewCollectionItem *item = gal_view_collection_get_view_item (dialog->collection, row);
+
+		if (item) {
+			gtk_widget_set_sensitive (glade_xml_get_widget (dialog->gui, "button-delete"),
+						  !item->built_in);
+			if (GAL_VIEW_GET_CLASS (item->view)->edit)
+				gtk_widget_set_sensitive (glade_xml_get_widget (dialog->gui, "button-modify"),
+						item->built_in);
+			else
+				gtk_widget_set_sensitive (glade_xml_get_widget (dialog->gui, "button-modify"),
+						!item->built_in);
+		}
+	}
+}
+
+static void
+etable_selection_change (ETable *etable, GalDefineViewsDialog *dialog)
+{
+	e_table_selected_row_foreach (etable, (EForeachFunc) etable_selection_change_forall_cb, dialog);
+}
+
+static void
+dialog_response (GalDefineViewsDialog *dialog, int response_id, gpointer data)
+{
+	gal_view_collection_save (dialog->collection);
+}	
+
 static void
 gal_define_views_dialog_init (GalDefineViewsDialog *dialog)
 {
 	GladeXML *gui;
 	GtkWidget *widget;
 	GtkWidget *etable;
+	gchar *filename = g_build_filename (GAL_GLADEDIR,
+					    "gal-define-views.glade",
+					    NULL);
 
 	dialog->collection = NULL;
 
-	gui = glade_xml_new (GAL_GLADEDIR "/gal-define-views.glade", NULL, E_I18N_DOMAIN);
+	gui = glade_xml_new (filename, NULL, E_I18N_DOMAIN);
+	g_free (filename);
 	dialog->gui = gui;
 
 	widget = glade_xml_get_widget(gui, "table-top");
@@ -230,8 +266,7 @@ gal_define_views_dialog_init (GalDefineViewsDialog *dialog)
 	gtk_widget_unref(widget);
 
 	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				GTK_STOCK_OK, GTK_RESPONSE_OK,
+				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 				NULL);
 
 	gdvd_connect_signal(dialog, "button-new",    "clicked", G_CALLBACK(gdvd_button_new_callback));
@@ -248,10 +283,16 @@ gal_define_views_dialog_init (GalDefineViewsDialog *dialog)
 		g_object_set(dialog->model,
 			     "collection", dialog->collection,
 			     NULL);
+		g_signal_connect (e_table_scrolled_get_table (E_TABLE_SCROLLED (etable)),
+				  "selection_change",
+				  G_CALLBACK (etable_selection_change), dialog);
 		gtk_widget_show_all (etable);
 	}
 
 	gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, TRUE, FALSE);
+
+	g_signal_connect (dialog, "response", G_CALLBACK (dialog_response), NULL);
+
 }
 
 static void
