@@ -22,47 +22,63 @@
  */
 
 #include <config.h>
-#include "e-util.h"
 
-#include <glib.h>
-#include <gtk/gtkobject.h>
+#include <stdlib.h>
+#include <stdio.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <unistd.h>
 #include <ctype.h>
-#include <sys/stat.h>
+#include <math.h>
 #include <string.h>
 #include <locale.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <time.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
-#if 0
-#include <libgnomevfs/gnome-vfs.h>
-#endif
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <gtk/gtk.h>
+#include <libgnome/gnome-util.h>
+
+#include "e-i18n.h"
+#include "e-util.h"
+#include "e-util-private.h"
 
 int
-g_str_compare(const void *x, const void *y)
+g_str_compare (const void *x, const void *y)
 {
 	if (x == NULL || y == NULL) {
 		if (x == y)
 			return 0;
 		else
 			return x ? -1 : 1;
-	} 
-		
-	return strcmp(x, y);
+	}
+	
+	return strcmp (x, y);
 }
 
 int
-g_int_compare(const void *x, const void *y)
+g_collate_compare (const void *x, const void *y)
 {
-  if ( GPOINTER_TO_INT(x) < GPOINTER_TO_INT(y) )
-    return -1;
-  else if ( GPOINTER_TO_INT(x) == GPOINTER_TO_INT(y) )
-    return 0;
-  else
-    return 1;
+	if (x == NULL || y == NULL) {
+		if (x == y)
+			return 0;
+		else
+			return x ? -1 : 1;
+	}
+	
+	return g_utf8_collate (x, y);
+}
+
+int
+g_int_compare (const void *x, const void *y)
+{
+	if (GPOINTER_TO_INT (x) < GPOINTER_TO_INT (y))
+		return -1;
+	else if (GPOINTER_TO_INT (x) == GPOINTER_TO_INT (y))
+		return 0;
+	else
+		return 1;
 }
 
 char *
@@ -88,7 +104,7 @@ e_free_object_list (GList *list)
 	GList *p;
 
 	for (p = list; p != NULL; p = p->next)
-		gtk_object_unref (GTK_OBJECT (p->data));
+		g_object_unref (p->data);
 
 	g_list_free (list);
 }
@@ -99,7 +115,7 @@ e_free_object_slist (GSList *list)
 	GSList *p;
 
 	for (p = list; p != NULL; p = p->next)
-		gtk_object_unref (GTK_OBJECT (p->data));
+		g_object_unref (p->data);
 
 	g_slist_free (list);
 }
@@ -138,7 +154,7 @@ e_read_file(const char *filename)
 	int bytes;
 	char *ret_val;
 
-	fd = open(filename, O_RDONLY);
+	fd = g_open(filename, O_RDONLY, 0);
 	if (fd == -1)
 		return NULL;
 	bytes = read(fd, buffer, BUFF_SIZE);
@@ -182,7 +198,7 @@ e_write_file(const char *filename, const char *data, int flags)
 	int fd;
 	int length = strlen(data);
 	int bytes;
-	fd = open(filename, flags, 0666);
+	fd = g_open(filename, flags | O_WRONLY, 0666);
 	if (fd == -1)
 		return errno;
 	while (length > 0) {
@@ -210,7 +226,7 @@ e_write_file_mkstemp(char *filename, const char *data)
 	int fd;
 	int length = strlen(data);
 	int bytes;
-	fd = mkstemp (filename);
+	fd = g_mkstemp (filename);
 	if (fd == -1)
 		return errno;
 	while (length > 0) {
@@ -221,7 +237,7 @@ e_write_file_mkstemp(char *filename, const char *data)
 		} else {
 			if (errno != EINTR && errno != EAGAIN) {
 				int save_errno = errno;
-	close(fd);
+				close(fd);
 				return save_errno;
 			}
 		}
@@ -248,19 +264,37 @@ e_mkdir_hier(const char *path, mode_t mode)
 {
 	char *copy, *p;
 
-	p = copy = g_strdup (path);
+	if (g_path_is_absolute (path)) {
+		p = copy = g_strdup (path);
+	} else {
+		gchar *current_dir = g_get_current_dir();
+		p = copy = g_build_filename (current_dir, path, NULL);
+		g_free (current_dir);
+	}
+
+	p = g_path_skip_root (p);
 	do {
-		p = strchr (p + 1, '/');
+		char *p1 = strchr (p, '/');
+#ifdef G_OS_WIN32
+		{
+			char *p2 = strchr (p, '\\');
+			if (p1 == NULL ||
+			    (p2 != NULL && p2 < p1))
+				p1 = p2;
+		}
+#endif
+		p = p1;
 		if (p)
 			*p = '\0';
-		if (access (copy, F_OK) == -1) {
-			if (mkdir (copy, mode) == -1) {
+		if (!g_file_test (copy, G_FILE_TEST_IS_DIR)) {
+			if (g_mkdir (copy, mode) == -1) {
 				g_free (copy);
 				return -1;
 			}
 		}
-		if (p)
-			*p = '/';
+		if (p) {
+			*p++ = '/';
+		}
 	} while (p);
 
 	g_free (copy);
@@ -315,7 +349,6 @@ e_read_uri(const char *uri)
 #include "e-marshal.h"
 #include "e-marshal.c"
 
-
 gchar**
 e_strsplit (const gchar *string,
 	    const gchar *delimiter,
@@ -393,16 +426,24 @@ e_strstrcase (const gchar *haystack, const gchar *needle)
 	return NULL;
 }
 
+/* This only makes a filename safe for usage as a filename.  It still may have shell meta-characters in it. */
 void
 e_filename_make_safe (gchar *string)
 {
-	gchar *p;
+	gchar *p, *ts;
+	gunichar c;
 	
 	g_return_if_fail (string != NULL);
-	
-	for (p = string; *p; p++) {
-		if (!isprint ((unsigned char)*p) || strchr (" /'\"`&();|<>${}!", *p))
-			*p = '_';
+	p = string;
+
+	while(p && *p) {
+		c = g_utf8_get_char (p);
+		ts = p;
+		p = g_utf8_next_char (p);
+		if (!g_unichar_isprint(c) || ( c < 0xff && strchr (" /'\"`&();|<>$%{}!", c&0xff ))) {
+			while (ts<p) 	
+				*ts++ = '_';
+		}
 	}
 }
 
@@ -478,10 +519,97 @@ e_format_number (gint number)
 	}
 }
 
+static gchar *
+do_format_number_as_float (double number)
+{
+	GList *iterator, *list = NULL;
+	struct lconv *locality;
+	gint char_length = 0;
+	gint group_count = 0;
+	guchar *grouping;
+	int last_count = 3;
+	int divider;
+	char *value;
+	char *value_iterator;
+	double fractional;
+
+	locality = localeconv();
+	grouping = locality->grouping;
+	while (number >= 1.0) {
+		char *group;
+		switch (*grouping) {
+		default:
+			last_count = *grouping;
+			grouping++;
+			/* Fall through */
+		case 0:
+			divider = epow10(last_count);
+			number /= divider;
+			fractional = modf (number, &number);
+			fractional *= divider;
+			fractional = floor (fractional);
+
+			if (number >= 1.0) {
+				group = g_strdup_printf("%0*d", last_count, (int) fractional);
+			} else {
+				group = g_strdup_printf("%d", (int) fractional);
+			}
+			break;
+		case CHAR_MAX:
+			divider = epow10(last_count);
+			number /= divider;
+			fractional = modf (number, &number);
+			fractional *= divider;
+			fractional = floor (fractional);
+
+			while (number >= 1.0) {
+				group = g_strdup_printf("%0*d", last_count, (int) fractional);
+
+				char_length += strlen(group);
+				list = g_list_prepend(list, group);
+				group_count ++;
+
+				divider = epow10(last_count);
+				number /= divider;
+				fractional = modf (number, &number);
+				fractional *= divider;
+				fractional = floor (fractional);
+			}
+
+			group = g_strdup_printf("%d", (int) fractional);
+			break;
+		}
+		char_length += strlen(group);
+		list = g_list_prepend(list, group);
+		group_count ++;
+	}
+
+	if (list) {
+		value = g_new(char, 1 + char_length + (group_count - 1) * strlen(locality->thousands_sep));
+
+		iterator = list;
+		value_iterator = value;
+
+		strcpy(value_iterator, iterator->data);
+		value_iterator += strlen(iterator->data);
+		for (iterator = iterator->next; iterator; iterator = iterator->next) {
+			strcpy(value_iterator, locality->thousands_sep);
+			value_iterator += strlen(locality->thousands_sep);
+
+			strcpy(value_iterator, iterator->data);
+			value_iterator += strlen(iterator->data);
+		}
+		e_free_string_list (list);
+		return value;
+	} else {
+		return g_strdup("0");
+	}
+}
+
 gchar *
 e_format_number_float (gfloat number)
 {
-	gint            int_part;
+	gfloat          int_part;
 	gint            fraction;
 	struct lconv   *locality;
 	gchar          *str_intpart;
@@ -491,8 +619,8 @@ e_format_number_float (gfloat number)
 
 	locality = localeconv();
 	
-	int_part = (int) number;
-	str_intpart = e_format_number (int_part);
+	int_part = floor (number);
+	str_intpart = do_format_number_as_float ((double) int_part);
 
 	if (!strcmp(locality->mon_decimal_point, "")) {
 		decimal_point = ".";
@@ -505,8 +633,7 @@ e_format_number_float (gfloat number)
 
 	if (fraction == 0) {
 		str_fraction = g_strdup ("00");
-	}
-	else {
+	} else {
 		str_fraction = g_strdup_printf ("%02d", fraction);
 	}
 
@@ -521,63 +648,11 @@ e_format_number_float (gfloat number)
 gboolean
 e_create_directory (gchar *directory)
 {
-	gchar *full_name;
-	gchar *position;
-	gchar *current_dir = g_get_current_dir();
-	struct stat info;
-	gboolean return_value = TRUE;
-
-	if (directory[0] == '/') {
-		full_name = g_malloc0 (strlen (directory) + 1);
-		strcpy (full_name, directory);
-	} else {
-		full_name = g_malloc0 (strlen (directory) + strlen (current_dir) + 2);
-		sprintf (full_name, "%s/%s", current_dir, directory);
-	}
-
-	if ((position = strrchr (full_name, '/')) == full_name) {
-		if (stat (full_name, &info)) {
-			switch (errno) {
-			case ENOENT:
-				if (mkdir (full_name, 0777)) {
-					switch (errno) {
-					default:
-						return_value = FALSE;
-						break;
-					}
-				}
-				break;
-			default:
-				return_value = FALSE;
-				break;
-			}
-		}
-	} else {
-		*position = 0;
-		e_create_directory (full_name);
-		*position = '/';
-		if (stat (full_name, &info)) {
-			switch (errno) {
-			case ENOENT:
-				if (mkdir (full_name, 0777)) {
-					switch (errno) {
-					default:
-						return_value = FALSE;
-						break;
-					}
-				}
-				break;
-			default:
-				return_value = FALSE;
-				break;
-			}
-		}
-	}
-
-	g_free (current_dir);
-	g_free (full_name);
-
-	return (return_value);
+	gint ret_val = e_mkdir_hier (directory, 0777);
+	if (ret_val == -1)
+		return FALSE;
+	else
+		return TRUE;
 }
 
 
@@ -712,6 +787,42 @@ size_t e_strftime(char *s, size_t max, const char *fmt, const struct tm *tm)
 #endif
 }
 
+size_t 
+e_utf8_strftime(char *s, size_t max, const char *fmt, const struct tm *tm)
+{
+	size_t sz, ret;
+	char *locale_fmt, *buf;
+
+	locale_fmt = g_locale_from_utf8(fmt, -1, NULL, &sz, NULL);
+	if (!locale_fmt)
+		return 0;
+
+	ret = e_strftime(s, max, locale_fmt, tm);
+	if (!ret) {
+		g_free (locale_fmt);
+		return 0;
+	}
+
+	buf = g_locale_to_utf8(s, ret, NULL, &sz, NULL);
+	if (!buf) {
+		g_free (locale_fmt);
+		return 0;
+	}
+
+	if (sz >= max) {
+		char *tmp = buf + max - 1;
+		tmp = g_utf8_find_prev_char(buf, tmp);
+		if (tmp)
+			sz = tmp - buf;
+		else
+			sz = 0;
+	}
+	memcpy(s, buf, sz);
+	s[sz] = '\0';
+	g_free(locale_fmt);
+	g_free(buf);
+	return sz;
+}
 
 /**
  * Function to do a last minute fixup of the AM/PM stuff if the locale
@@ -774,6 +885,43 @@ size_t e_strftime_fix_am_pm(char *s, size_t max, const char *fmt, const struct t
 	return(ret);
 }
 
+size_t 
+e_utf8_strftime_fix_am_pm(char *s, size_t max, const char *fmt, const struct tm *tm)
+{
+	size_t sz, ret;
+	char *locale_fmt, *buf;
+
+	locale_fmt = g_locale_from_utf8(fmt, -1, NULL, &sz, NULL);
+	if (!locale_fmt)
+		return 0;
+
+	ret = e_strftime_fix_am_pm(s, max, locale_fmt, tm);
+	if (!ret) {
+		g_free (locale_fmt);
+		return 0;
+	}
+
+	buf = g_locale_to_utf8(s, ret, NULL, &sz, NULL);
+	if (!buf) {
+		g_free (locale_fmt);
+		return 0;
+	}
+
+	if (sz >= max) {
+		char *tmp = buf + max - 1;
+		tmp = g_utf8_find_prev_char(buf, tmp);
+		if (tmp)
+			sz = tmp - buf;
+		else
+			sz = 0;
+	}
+	memcpy(s, buf, sz);
+	s[sz] = '\0';
+	g_free(locale_fmt);
+	g_free(buf);
+	return sz;
+}
+
 /**
  * e_flexible_strtod:
  * @nptr:    the string to convert to a numeric value.
@@ -1051,3 +1199,44 @@ e_strdup_append_strings (gchar *first_string, ...)
 
 	return buffer;
 }
+
+gchar **
+e_strdupv (const gchar **str_array)
+{
+	if (str_array) {
+		gint i;
+		gchar **retval;
+
+		i = 0;
+		while (str_array[i])
+			i++;
+          
+		retval = g_new (gchar*, i + 1);
+
+		i = 0;
+		while (str_array[i]) {
+			retval[i] = g_strdup (str_array[i]);
+			i++;
+		}
+		retval[i] = NULL;
+
+		return retval;
+	} else {
+		return NULL;
+	}
+}
+
+char *
+e_gettext (const char *msgid)
+{
+	static gboolean initialized = FALSE;
+
+	if (!initialized) {
+		bindtextdomain (E_I18N_DOMAIN, GAL_LOCALEDIR);
+		bind_textdomain_codeset (E_I18N_DOMAIN, "UTF-8");
+		initialized = TRUE;
+	}        
+
+	return dgettext (E_I18N_DOMAIN, msgid);
+}
+
